% $Author: Vincent Drouard $    $Date: Apr 1, 2016$    $Revision: 1.0 $
%                               
% Copyright: Perception Team, INRIA-Grenoble
% Email: vincent.drouard@inria.fr

% The Head Pose algorithm is described in this paper:      
%  (1) V. Drouard, S. Ba, G. Evangelidis, A. Deleforge, and R. Horaud ,  "Head Pose Estimation via Probabilistic High-Dimensional Regression", IEEE International Conference on Image Processing (ICIP), 2015.
function h = HOG(img,nbBins,cells,blocks)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Description: Compute the HoG of an image
%%%%%
%%% INPUT
% img: gray level image
% nbBins: number of orientation bins for the the HoG
% cells: 1x2 matrix, size of a cell in pixels
% blocks; 1x2 matrix, size of a block in cells
%%% OUTPUT
% h: column vector with the corresponding HoG 
%%%%%%%%%%%%%%%%%%%%%%%%%
    shift = 1;
    filX=[-1 0 1]; % discrete derivative mask x direction
    filY=filX'; % discrete derivative mask y direction
    
    Gx = imfilter(double(img),filX); % compute gradient x direction
    Gy = imfilter(double(img),filY); % compute gradient y direction
    Gmag=sqrt(Gx.^2+Gy.^2); % magnitude of the gradient 
    Gdir=atan2(Gy,Gx); % orientation of the gradient

    Gdir = (Gdir<0).*(Gdir+pi) + (Gdir>=0).*Gdir; % unsigned orientation
    
    delta = pi/nbBins; % bin size
    GdirQ = mod(floor(Gdir/delta),nbBins)+1; % orientation quantization

    % number of cells
    [u,v]=size(img);
    nbcellsx=ceil(u/cells(1));
    nbcellsy=ceil(v/cells(2));
    
    % number of blocks	
    nbBx=floor((nbcellsx-blocks(1))/shift)+1;
    nbBy=floor((nbcellsy-blocks(2))/shift)+1;
    
    %block shift in pixels
    sx=cells(1)*shift;
    sy=cells(2)*shift;

    
    x=0;
    y=0;
    h=zeros(nbBins*blocks(1)*blocks(2),nbBx*nbBy);
    e = 10^-8;
    for i = 1:nbBx*nbBy % loop over blocks
        GdirQb = GdirQ(x+1:x+cells(1)*blocks(1),y+1:y+cells(2)*blocks(2)); % extract subgradient quantized orientation
        Gmagb = Gmag(x+1:x+cells(1)*blocks(1),y+1:y+cells(2)*blocks(2)); % extract subgradient magnitude
        xb=0;yb=0;sxb=cells(1);syb=cells(2);
        for j = 1:blocks(1)*blocks(2) % loop over cells in block
            GdirQbc = GdirQb(xb+1:xb+sxb,yb+1:yb+syb); % extract subgradient quantized orientation
            Gmagbc = Gmagb(xb+1:xb+sxb,yb+1:yb+syb); % extract subgradient magnitude
            for k = 1:nbBins % loop over all discrete orientation
                h(k+(j-1)*nbBins,i) = sum((GdirQbc(:)==k).*Gmagbc(:));
            end
            clb = (xb + sxb + cells(1)<=blocks(1)*cells(1));
            yb = yb + (1-clb)*syb;
            xb = clb*(xb + sxb) + (1-clb)*0;
        end
        h(:,i) = h(:,i)/sqrt(sum(h(:,i).^2)+e^2); % histogram normalization
        
        cl = (x + sx + blocks(1)*cells(1)<=v);
        y = y + (1-cl)*sy;
        x = cl*(x + sx) + (1-cl)*0;
    end
    h=reshape(h,size(h,1)*size(h,2),1); % vectorize





end   

